#include<stdio.h>
#include<unistd.h>
#include<stdlib.h>
#include<time.h>
#include<fcntl.h>
#include<string.h>
#include<signal.h>
#include<errno.h>
#include<sys/stat.h>

#define EXIT_WITH_CLEAN(code)            do { exit(unlink(Z_PID_FILE) == -1?1:code); } while(0);
#define EXIT_WITH_ERRNO(format, args...) do { \
        printf(format":(%d) %s\n", ##args, errno, strerror(errno)); EXIT_WITH_CLEAN(1); \
    } while(0)

char Z_CNF_FILE[512]="/etc/sptest/sptest.conf";
char Z_PID_FILE[512]="/var/run/sptest/sptest.pid";
char Z_LOG_FILE[512]="/tmp/sptest.log";
int  B_SET_DEAMON   =0;

void parse_opts(int argc, char **argv)
{
    for (int c = 0; (c = getopt (argc, argv, "dc:p:f:")) != -1; )
    {
        switch (c)
        {
            case 'd':
                B_SET_DEAMON = 1;
                break;
            case 'c':
                snprintf(Z_CNF_FILE, sizeof(Z_CNF_FILE), "%s", optarg);
                break;
            case 'f':
                snprintf(Z_LOG_FILE, sizeof(Z_LOG_FILE), "%s", optarg);
                break;
            case 'p':
                snprintf(Z_PID_FILE, sizeof(Z_PID_FILE), "%s", optarg);
                break;
            case '?':
                fprintf (stderr, "Unknown option character `\\x%x'.\n", optopt);
                return ;
            default:
                abort();
        }
    }
}

void write_pidfile(const char *zPidFile, pid_t iPid)
{
    char zPidPath[512];
    memset(zPidPath, 0, sizeof(zPidPath));

    int iLen = strrchr(zPidFile, '/') - zPidFile;

    snprintf(zPidPath, sizeof(zPidPath), "%.*s", iLen, zPidFile);

    if (access(zPidPath, F_OK) != 0)
    {
        if (mkdir(zPidPath, S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH) != 0)
        {
            EXIT_WITH_ERRNO("Can't create dir %s for pidfile", zPidPath);
        }
    }

    FILE *fp;
    if ((fp = fopen(Z_PID_FILE, "w")) == NULL)
    {
        EXIT_WITH_ERRNO("Can't open pidfile %s for writing", Z_PID_FILE);
    }

    fprintf(fp, "%d\n", iPid); 
    fclose(fp);
}

void set_daemon(void)
{
    pid_t pid;
    pid = fork();
    if (pid == -1)
        EXIT_WITH_ERRNO("fork error");
    if (pid >   0)
        exit(EXIT_SUCCESS);

    if (setsid() == -1)
        EXIT_WITH_ERRNO("SETSID ERROR");

    chdir("/");

    close(STDIN_FILENO);
    close(STDOUT_FILENO);
    close(STDERR_FILENO);
    open("/dev/null", O_RDONLY); /* 0 = stdin  */
    open("/dev/null", O_WRONLY); /* 1 = stdout */
    open("/dev/null", O_RDWR);   /* 2 = stderr */

    umask(0);

    return;
}

void sig_handle(int sig)
{
    printf("sig_handle:%d\n", sig);

    EXIT_WITH_CLEAN(0);
}

int main(int argc, char **argv)
{
    parse_opts(argc, argv);

    if (B_SET_DEAMON == 1)
    {
        set_daemon();
    }

    write_pidfile(Z_PID_FILE, getpid());

    signal(SIGUSR1, sig_handle);

    {
        int iFile = open(Z_LOG_FILE, O_WRONLY|O_CREAT|O_APPEND, 0644);
        if (iFile == -1)
        {
            EXIT_WITH_ERRNO("Can't open logfile %s for writing", Z_LOG_FILE);
        }

        for(time_t t = time(0); 1; t = time(0))
        {
            char *zBuff = asctime(localtime(&t));
            write(iFile, zBuff, strlen(zBuff));

            sleep(60);
        }

        close(iFile);
    }

    return 0;
}
